# -*- coding: utf-8 -*-

# import sys
# import traceback
from synapseqrcodes import synapse_qrcode
from synapseqrcodes import verify_parameter
try:
    from synapselabels import __version__

    assert __version__
except ImportError:
    __version__ = "UNKNOWN"
try:
    from gi import require_version
except ImportError:
    print("ERROR: pygobject is not installed.")
    raise
try:
    require_version("Gtk", "3.0")
except ValueError:
    print("ERROR: gtk3 development files are not installed.")
    raise
except AttributeError:
    print("pygobject version too old.")
    raise
try:
    from gi.repository import Gtk
    # from gi.repository import Gdk
    # from gi.repository import GLib
    # from gi.repository import GObject
except ImportError:
    print("ERROR: pygobject is not installed.")
    raise


def gtk_input(
        parent,
        parameter,
        record,
        records,
        default='',
        autoincrement_option=False,
        fixed_option=False,
        autogenerate_option=False,
):
    """
    Display a dialog with a text entry.
    Returns the text, or None if canceled.
    """
    message = (
        "Please Enter or Scan\n<b><big>%s</big></b>\n(%d of %d)" %
        (parameter, record + 1, records)
    )
    d = Gtk.MessageDialog(
        parent,
        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
        Gtk.MessageType.QUESTION,
        Gtk.ButtonsType.OK_CANCEL,
        # message
    )
    vbox = d.get_message_area()
    d.set_markup(message)
    entry = Gtk.Entry()
    entry.set_text(default)
    entry.show()
    if autoincrement_option:
        auto_inc = Gtk.CheckButton(label='Autoincrement')
        auto_inc.show()
        vbox.pack_end(auto_inc, True, True, 0)
    if autogenerate_option:
        auto_gen = Gtk.CheckButton(label='Autogenerate')
        auto_gen.show()
        vbox.pack_end(auto_gen, True, True, 0)
    if fixed_option:
        fixed = Gtk.CheckButton(label='Fixed')
        fixed.show()
        vbox.pack_end(fixed, True, True, 0)
    vbox.pack_end(entry, True, True, 0)
    entry.connect('activate', lambda _: d.response(Gtk.ResponseType.OK))
    d.set_default_response(Gtk.ResponseType.OK)
    r = d.run()
    text = entry.get_text().decode('utf8')
    d.destroy()
    if r == Gtk.ResponseType.OK:
        return (
            text,
            auto_gen.get_active() if autogenerate_option else
            auto_inc.get_active() if autoincrement_option else
            fixed.get_active() if fixed_option else
            False
        )


def hexadecimal_increment(hex_string):
    if len(hex_string) == 0:
        return '1'
    last = 1 + int(hex_string[-1:], 16)
    if last <= 15:
        return hex_string[:-1].upper() + ('%X' % last)
    return hexadecimal_increment(hex_string[:-1]) + '0'


def decimal_increment(dec_string):
    if len(dec_string) == 0:
        return '1'
    last = 1 + int(dec_string[-1:])
    if last <= 9:
        return dec_string[:-1] + str(last)
    return decimal_increment(dec_string[:-1]) + '0'


class Generator():
    """Generate records"""
    def __init__(self):
        """Initialize generator"""
        self.autoincremented_params = {}
        self.fixed_params = {}
        self.autogenerated_params = []
        self.scanned_qr_code = {}

    def generate_n_records(self, records):
        # print('records', records)
        parameters = self.get_parameters()
        retval = '\t'.join(parameters) + '\n'
        # reset the generators
        self.autoincremented_params = {}
        self.fixed_params = {}
        self.autogenerated_params = []
        self.scanned_qr_code = {}
        for record in range(records):
            more = record < (records - 1)
            retval += '\t'.join(
                [
                    self.generate_parameter(
                        parameter,
                        more,
                        record,
                        records,
                    ) for parameter in parameters
                ]
            ) + '\n'
        return retval

    def generate_parameter(self, parameter, more, record, records):
        if parameter in ('date code',):
            # prompt operator for optionally fixed parameter
            if parameter in self.fixed_params:
                new_param = self.fixed_params[parameter]
            else:
                while True:
                    new_param, fixed = gtk_input(
                        self.window,
                        parameter,
                        record,
                        records,
                        default=self.entry_date_code.get_text(),
                        fixed_option=True,
                    )
                    if new_param is None:
                        return None
                    new_param = new_param.strip()
                    try:
                        self.scanned_qr_code = synapse_qrcode(new_param)
                        new_param = self.scanned_qr_code[parameter]
                    except ValueError:
                        # silently pass,
                        # if it was not a Synapse QR code it is not a problem
                        pass
                    try:
                        verify_parameter(parameter, new_param)
                        break
                    except ValueError as error:
                        print(error)
                if fixed:
                    self.fixed_params[parameter] = new_param
            return new_param
        elif parameter in ('HW version',):
            # prompt operator for optionally fixed parameter
            if parameter in self.fixed_params:
                new_param = self.fixed_params[parameter]
            else:
                while True:
                    new_param, fixed = gtk_input(
                        self.window,
                        parameter,
                        record,
                        records,
                        fixed_option=True,
                    )
                    if new_param is None:
                        return None
                    new_param = new_param.strip()
                    try:
                        self.scanned_qr_code = synapse_qrcode(new_param)
                        new_param = self.scanned_qr_code[parameter]
                    except ValueError:
                        # silently pass,
                        # if it was not a Synapse QR code it is not a problem
                        pass
                    try:
                        verify_parameter(parameter, new_param)
                        break
                    except ValueError as error:
                        print(error)
                if fixed:
                    self.fixed_params[parameter] = new_param
            return new_param
        elif parameter in ('production order number', 'batch size',):
            # prompt operator for forcefully fixed parameter
            if parameter in self.fixed_params:
                new_param = self.fixed_params[parameter]
            else:
                while True:
                    new_param, fixed = gtk_input(
                        self.window,
                        parameter,
                        record,
                        records,
                        default=(
                            self.scanned_qr_code[parameter]
                            if parameter in self.scanned_qr_code else ''
                        )
                    )
                    if new_param is None:
                        return None
                    new_param = new_param.strip()
                    try:
                        self.scanned_qr_code = synapse_qrcode(new_param)
                        new_param = self.scanned_qr_code[parameter]
                    except ValueError:
                        # silently pass,
                        # if it was not a Synapse QR code it is not a problem
                        pass
                    new_param = new_param.upper()
                    try:
                        verify_parameter(parameter, new_param)
                        break
                    except ValueError as error:
                        print(error)
                self.fixed_params[parameter] = new_param
            return new_param
        elif parameter in (
                'default password for user access',
                'default password for wifi access',
        ):
            # automatically generate parameter, no prompt
            if parameter in self.autogenerated_params:
                return self.password_generator.generate_password()
            else:
                while True:
                    new_param, auto_gen = gtk_input(
                        self.window,
                        parameter,
                        record,
                        records,
                        autogenerate_option=True,
                        default=(
                            self.scanned_qr_code[parameter]
                            if parameter in self.scanned_qr_code else ''
                        )
                    )
                    if new_param is None:
                        return None
                    if auto_gen:
                        self.autogenerated_params.append(parameter)
                        return self.password_generator.generate_password()
                    new_param = new_param.strip()
                    try:
                        self.scanned_qr_code = synapse_qrcode(new_param)
                        new_param = self.scanned_qr_code[parameter]
                    except ValueError:
                        # silently pass,
                        # if it was not a Synapse QR code it is not a problem
                        pass
                    try:
                        verify_parameter(parameter, new_param)
                        break
                    except ValueError as error:
                        print(error)
            return new_param
        elif parameter in (
                'ethernet mac address',
                'snap mac address',
                'wifi mac address',
                'snap node address',
                'modem meid',
        ):
            # prompt operator for optionally autoincremented
            # hexadecimal parameter
            if parameter in self.autoincremented_params:
                new_param = hexadecimal_increment(
                    self.autoincremented_params[parameter]
                )
                self.autoincremented_params[parameter] = new_param
            else:
                while True:
                    new_param, auto_inc = gtk_input(
                        self.window,
                        parameter,
                        record,
                        records,
                        autoincrement_option=more,
                        default=(
                            self.scanned_qr_code[parameter]
                            if parameter in self.scanned_qr_code else ''
                        )
                    )
                    if new_param is None:
                        return None
                    new_param = new_param.strip()
                    # print('new_param:', new_param)
                    try:
                        self.scanned_qr_code = synapse_qrcode(new_param)
                        print('self.scanned_qr_code:', self.scanned_qr_code)
                        new_param = self.scanned_qr_code[parameter]
                        print(parameter, new_param)
                    except ValueError:
                        # silently pass,
                        # if it was not a Synapse QR code it is not a problem
                        pass
                    new_param = new_param.upper()
                    try:
                        verify_parameter(parameter, new_param)
                        break
                    except ValueError as error:
                        print(error)
                if auto_inc:
                    self.autoincremented_params[parameter] = new_param
            return new_param
        elif parameter in ('sim iccid', 'sequential code',):
            # prompt operator for optionally autoincremented
            # decimal parameter
            if parameter in self.autoincremented_params:
                new_param = decimal_increment(
                    self.autoincremented_params[parameter]
                )
                self.autoincremented_params[parameter] = new_param
            else:
                while True:
                    new_param, auto_inc = gtk_input(
                        self.window,
                        parameter,
                        record,
                        records,
                        autoincrement_option=more,
                        default=(
                            self.scanned_qr_code[parameter]
                            if parameter in self.scanned_qr_code else ''
                        )
                    )
                    if new_param is None:
                        return None
                    new_param = new_param.strip()
                    try:
                        self.scanned_qr_code = synapse_qrcode(new_param)
                        new_param = self.scanned_qr_code[parameter]
                    except ValueError:
                        # silently pass,
                        # if it was not a Synapse QR code it is not a problem
                        pass
                    try:
                        verify_parameter(parameter, new_param)
                        break
                    except ValueError as error:
                        print(error)
                if auto_inc:
                    self.autoincremented_params[parameter] = new_param
            return new_param
        else:
            # TBD pop-up info dialog instead
            raise ValueError(
                "cannot generate parameter: %s" %
                repr(parameter)
            )
